#include <winver.h>

#define SMPLAYER_FILEVERSION 18,10,0,0
#define SMPLAYER_PRODVERSION "18.10.0.0"

VS_VERSION_INFO     VERSIONINFO
FILEVERSION         SMPLAYER_FILEVERSION
PRODUCTVERSION      SMPLAYER_FILEVERSION
FILEOS              VOS_NT_WINDOWS32
FILETYPE            VFT_APP
{
  BLOCK "StringFileInfo"
  {
    BLOCK "000004b0"
    {
      VALUE "CompanyName", "Ricardo Villalba\0"
      #ifdef PORTABLE_APP
      VALUE "FileDescription", "SMPlayer (Portable Edition)\0"
      #else
      VALUE "FileDescription", "SMPlayer\0"
      #endif
      VALUE "FileVersion", SMPLAYER_PRODVERSION "\0"
      VALUE "InternalName", "smplayer\0"
      VALUE "LegalCopyright", "Copyright (c) Ricardo Villalba\0"
      VALUE "OriginalFilename", "smplayer.exe\0"
      #if defined(_WIN64)
      VALUE "ProductName", "SMPlayer for Windows (64-bit)\0"
      #elif defined(_WIN32) && !defined(_WIN64)
      VALUE "ProductName", "SMPlayer for Windows (32-bit)\0"
      #endif
      VALUE "ProductVersion", SMPLAYER_PRODVERSION "\0"
    }
  }
  BLOCK "VarFileInfo"
  {
    VALUE "Translation", 0, 0x04b0
  }
}

IDI_ICON1   ICON    DISCARDABLE   "smplayer.ico"
IDI_ICON2   ICON    DISCARDABLE   "smplayer_associations.ico"
