/*  smplayer, GUI front-end for mplayer.
    Copyright (C) 2006-2018 Ricardo Villalba <rvm@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef LINKS_H
#define LINKS_H

#define URL_HOMEPAGE "http://www.smplayer.info"
#define URL_FORUM "http://forum.smplayer.info"
#define URL_TRANSLATORS URL_HOMEPAGE"/translators.php"
#define URL_FIRST_STEPS URL_HOMEPAGE"/first-steps.php"
#define URL_FAQ URL_HOMEPAGE"/faq.php"
#define URL_CHANGES URL_HOMEPAGE"/changes.php"
#define URL_THANK_YOU URL_HOMEPAGE"/thank-you.php"
#define URL_OPENSSL_INFO URL_HOMEPAGE"/openssl.php"
#define URL_DONATE URL_HOMEPAGE"/donate.php"
#define URL_CHROMECAST "http://chromecast.smplayer.info"
#define URL_YT_CODE "http://updates.smplayer.info/yt.js"
#define URL_VERSION_INFO "http://updates.smplayer.info/version_info.ini"

#endif
