/*  smplayer, GUI front-end for mplayer.
    Copyright (C) 2006-2018 Ricardo Villalba <rvm@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "inputurl.h"
#include "images.h"
#include "mylineedit.h"

InputURL::InputURL( QWidget* parent, Qt::WindowFlags f ) 
	: QDialog(parent, f)
{
	setupUi(this);

	setMinimumSize( QSize(500,140) );
	setMaximumSize( QSize(600,170) );
	//layout()->setSizeConstraint(QLayout::SetFixedSize);

	url_icon->setPixmap( Images::icon("url_big", 48) );
	url_edit->setFocus();

	MyLineEdit * edit = new MyLineEdit(this);
	url_edit->setLineEdit(edit);
}

InputURL::~InputURL() {
}

void InputURL::setURL(QString url) {
	url_edit->addItem(url);
}

QString InputURL::url() {
	return url_edit->currentText().trimmed();
}

#include "moc_inputurl.cpp"
